@@ -3,8 +3,9 @@ package _common;
 # common parts for Struct::Path tests
 
 use parent qw(Exporter);
+use Data::Dumper;
 
-our @EXPORT_OK = qw($s_array $s_hash $s_mixed);
+our @EXPORT_OK = qw($s_array $s_hash $s_mixed t_dump);
 
 our $s_array = [ 3, 1, 5, [9, [13], 7], 11];
 
@@ -26,4 +27,9 @@ our $s_mixed = {
     'c' => 'vc',
 };
 
+# return neat one-line string of perl serialized structure
+sub t_dump {
+    return Data::Dumper->new([shift])->Terse(1)->Sortkeys(1)->Quotekeys(0)->Indent(0)->Deepcopy(1)->Dump();
+}
+
 1;
